/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.1.0
 * Date               : 2024/08/01
 * Description        : Main program body.
 *******************************************************************************/
#include "debug.h"
#include "CH392INC.h"
#include "CH392.H"

#define Query392Interrupt() GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_1)

#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE) /* SEL = 1 */
#include "CH392UART.h"
#elif (CH392_OP_INTERFACE_MODE == CH392_SPI_MODE) /* SEL = 0 */
#include "CH392SPI_HW.h"
#else
#error "Please Select Correct Communication Interface "
#endif

/**********************************************************************************/
/* Include command file */
#include "CH392CMD.h"

/* Common variable definition */
UINT8 MyBuffer[4096];       /* data buffer */
struct _SOCK_INF SockInf;   /* Save the socket information */
struct _CH392_SYS CH392Inf; /* Save the CH392 information */

/* CH392 Related definition */
const UINT8 CH392IPAddr[4] = {192, 168, 1, 101}; /* CH392 IP  */
const UINT8 CH392GWIPAddr[4] = {192, 168, 1, 1}; /* CH392 gateway */
const UINT8 CH392IPMask[4] = {255, 255, 255, 0}; /* CH392 mask */

/* Socket Related definition*/
const UINT8 Socket0DesIP[4] = {192, 168, 1, 123}; /* Socket 0 des IP */
const UINT16 Socket0SourPort = 1000;              /* Socket 0 sour port */
const UINT16 Socket0DesPort = 1000;               /* Socket 0 des port */

/*
If using the SINT_STAT_SENBUF_FREE interrupt as the condition for determining whether to send a data packet,
the socket interrupt value can be set to SINT_STAT_SENBUF_FREE during initialization,
as there will be no send buffer idle interrupt before any data packet is sent.
*/
UINT8 sock_int_status = SINT_STAT_SENBUF_FREE;

/**********************************************************************************
 * Function Name  : mStopIfError
 * Description    : Display error code, and stop
 * Input          : iError
 * Output         : None
 * Return         : None
 **********************************************************************************/
void mStopIfError(UINT8 iError)
{
    if (iError == CMD_ERR_SUCCESS)
        return;                                /* Success */
    printf("Error: %02X\r\n", (UINT16)iError); /* printf error */
    while (1)
    {
        Delay_Ms(200);
        Delay_Ms(200);
    }
}

/**********************************************************************************
 * Function Name  : CH392SocketInterrupt
 * Description    : CH392 socket interrupt, called in global interrupt
 * Input          : sockindex
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392SocketInterrupt(UINT8 sockindex)
{
    UINT16 len, totallen;

    sock_int_status = CH392CMDGetSocketInt(sockindex); /* Gets the socket interrupt status */

    if (sock_int_status & SINT_STAT_RECV) /* Receive interruption */
    {
        printf("SINT_STAT_RECV\r\n");
        totallen = CH392CMDGetRecvLength(sockindex); /* Gets the length of data in the current buffer */

        while (totallen > 0)
        {
            /*The length of the sent data packet cannot be greater than 1472,
            so only data with a length less than 1472 is read and sent at one time.
            In practical use, the length of received data depends solely on the size of the receive buffer.*/
            if (totallen > 1472)
            {
                len = 1472;
            }
            else
            {
                len = totallen;
            }
            totallen -= len;
            CH392CMDGetRecvData(sockindex, len, MyBuffer);
            /*
            This code snippet is not necessary in certain cases.
            It should be used in situations where data packets are being sent exceptionally frequently
            to verify if the sending conditions are met.
            */
#if SEND_CONDITION == SEND_QUEUE_IS_IDLE
            while (CH392CMDGetIdleSendQueue() == 0)
            {
                printf("Send Idle Queue = 0\r\n");
                Delay_Us(100);
            };
#elif SEND_CONDITION == SEND_BUFFER_IS_NOT_EMPTY
            while ((sock_int_status[sockindex] & SINT_STAT_SENBUF_FREE) == 0)
            {
                Delay_Us(100);
                sock_int_status[sockindex] |= CH392CMDGetSocketInt(sockindex);
            };
            sock_int_status[sockindex] &= ~SINT_STAT_SENBUF_FREE;
#endif
            CH392CMDSendData(sockindex, MyBuffer, len);
        }
        sock_int_status &= ~SINT_STAT_RECV;
    }
    if (sock_int_status & SINT_STAT_SENBUF_FREE) /* The send buffer is free and can continue writing data to be sent */
    {
/*
If SINT_STAT_SENBUF_FREE is used as the condition for sending packets,
then the interrupt status can be cleared during packet transmission queries.
*/
#if SEND_CONDITION != SEND_BUFFER_IS_NOT_EMPTY
        sock_int_status &= ~SINT_STAT_SENBUF_FREE;
#endif
    }
    if (sock_int_status & SINT_STAT_SEND_OK) /* Send completion interrupt */
    {
        sock_int_status &= ~SINT_STAT_SEND_OK;
    }
}

/**********************************************************************************
 * Function Name  : CH392GlobalInterrupt
 * Description    : CH392 Global interrupt function
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void CH392GlobalInterrupt(void)
{
    UINT16 init_status;

    init_status = CH392CMDGetGlobIntStatus_392T();

    if (init_status & GINT_STAT_PHY_CHANGE) /* PHY change interrupt */
    {
        printf("Init status : GINT_STAT_PHY_CHANGE\r\n");
    }
    if (init_status & GINT_STAT_SOCK0)
    {
        CH392SocketInterrupt(0); /* socket 0 interrupt */
    }
}

/**********************************************************************************
 * Function Name  : CH392Init
 * Description    : Configure IP, GWIP, and MAC for the CH392, and initialize the CH392
 * Input          : None
 * Output         : None
 * Return         : Function execution result
 **********************************************************************************/
UINT8 CH392Init(void)
{
    UINT8 i;

    i = CH392CMDCheckExist(0x65);
    if (i != 0x9a)
        return CH392_ERR_UNKNOW; /* Test cmd , return 0XFA if it fails*/
                                 /* The value 0XFA is usually a hardware error or incorrect read/write timing */
    i = CH392CMDSetMACRaw();     /* initialize the CH392 */
    return i;
}
/**********************************************************************************
 * Function Name  : Gpio_Init
 * Description    : initialize GPIO
 * Input          : None
 * Output         : None
 * Return         : None
 **********************************************************************************/
void Gpio_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    // 392Interrupt
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    // 392Rset
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_SetBits(GPIOA, GPIO_Pin_4);
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    UINT8 i;
    Delay_Init();
    USART_Printf_Init(115200);
#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE) /* SEL = 1 */
    CH392_UARTPort_Init();
#elif (CH392_OP_INTERFACE_MODE == CH392_SPI_MODE) /* SEL = 0 */
    CH392_SPIPort_Init();
#endif

    Gpio_Init();
    CH392Reset();
    Delay_Ms(200);
#if (CH392_OP_INTERFACE_MODE == CH392_UART_MODE)
    CH392SetUartBaudRate(921600);
#endif
    printf("CH392EVT Test Demo\r\n");
    i = CH392CMDGetVer();
    printf("CH392VER : %2x\r\n", (UINT16)i);

    i = CH392Init(); /* Initialize the CH392 */
    mStopIfError(i);
    while (1)
    {                                              /* Wait for the CH392 Connect Ethernet*/
        if (CH392CMDGetPHYStatus() == PHY_DISCONN) /* Example Query whether the CH392 is connected */
        {
            Delay_Ms(200); /* If no, wait for 200MS and query again */
        }
        else
        {
            printf("CH392 Connect Ethernet\r\n"); /*When the CH392 is connected to the Ethernet, an interruption occurs */
            break;
        }
    }

    while (1)
    {
        if (Query392Interrupt() == 0)
        {
            CH392GlobalInterrupt();
        }
    }
}
